local key = ModPath .. '	' .. RequiredScript
if _G[key] then return else _G[key] = true end

if not Global.load_level then
	return
end

local table_insert = table.insert

local Perfometer = _G.Perfometer

local body_collisions_nr
local tasks_nr
local frames_nr
local delay_sum
local delay_nr
local delay_min
local delay_max
local old_sec

function Perfometer:reset_counters()
	body_collisions_nr = 0
	tasks_nr = 0
	frames_nr = 0
	delay_sum = 0
	delay_nr = 0
	delay_min = 9
	delay_max = 0
	old_sec = 0
	self.timer = TimerManager:wall()
end
Perfometer:reset_counters()

function Perfometer:start()
	if self.running then
		return
	end

	self:init_data()
	self:reset_counters()
	self.running = true
end

function Perfometer:stop()
	self.running = false
end

function Perfometer:update(t, dt)
	if self.running then
		frames_nr = frames_nr + 1

		local wt = self.timer:time()
		local sec = math.floor(wt)
		if sec ~= old_sec then
			local menemy = managers.enemy
			local ed = menemy._enemy_data
			local cops_nr = ed.nr_units
			local corpses_nr = ed.nr_corpses
			local civs_nr = table.size(menemy._civilian_data.unit_data)
			table_insert(self.npcs, cops_nr + civs_nr)
			table_insert(self.corpses, corpses_nr)
			table_insert(self.body_collisions, body_collisions_nr)
			table_insert(self.tasks, tasks_nr)
			table_insert(self.frames, frames_nr)
			table_insert(self.avg_delays, delay_nr > 0 and delay_sum / delay_nr or 0)
			table_insert(self.max_delays, delay_max > delay_min and delay_max or 0)

			body_collisions_nr = 0
			tasks_nr = 0
			frames_nr = 0
			delay_nr = 0
			delay_sum = 0
			delay_min = 9
			delay_max = 0
			old_sec = sec
		end
	end
end

local pm_original_setup_endframe = Setup.end_frame
function Setup:end_frame(t, dt)
	if _G.total_frames_nr then
		_G.total_frames_nr = _G.total_frames_nr + 1
	end
	Perfometer:update(t, dt)
	pm_original_setup_endframe(self, t, dt)
end

if not FullSpeedSwarm then
	local pm_original_enemymanager_update = EnemyManager.update
	function EnemyManager:update(t, dt)
		self._t = t
		pm_original_enemymanager_update(self, t, dt)
	end
end

local pm_original_enemymanager_executequeuedtask = EnemyManager._execute_queued_task
function EnemyManager:_execute_queued_task(i)
	tasks_nr = tasks_nr + 1

	local task_data = self._queued_tasks[i]
	if task_data.t then
		delay_nr = delay_nr + 1
		local delay = self._t - task_data.t
		delay_sum = delay_sum + delay
		delay_min = math.min(delay, delay_min)
		delay_max = math.max(delay, delay_max)
	end

	pm_original_enemymanager_executequeuedtask(self, i)
end

local pm_original_coreunitdamage_bodycollisioncallback = CoreUnitDamage.body_collision_callback
function CoreUnitDamage.body_collision_callback(...)
	body_collisions_nr = body_collisions_nr + 1
	pm_original_coreunitdamage_bodycollisioncallback(...)
end

local pm_original_civilianlogicflee_runawayfromalert = CivilianLogicFlee._run_away_from_alert
function CivilianLogicFlee._run_away_from_alert(data, ...)
	data.t = TimerManager:game():time() -- don't screw my pretty graph with t from 3 sec ago!
	pm_original_civilianlogicflee_runawayfromalert(data, ...)
end

local pm_original_groupaistatebase_getbalancingmultiplier = GroupAIStateBase._get_balancing_multiplier
function GroupAIStateBase:_get_balancing_multiplier(balance_multipliers)
	if Perfometer.settings.full_team_balance then
		return balance_multipliers[4]
	end

	return pm_original_groupaistatebase_getbalancingmultiplier(self, balance_multipliers)
end

core:module('CoreGameStateMachine')

local pm_original_gamestatemachine_changestate = GameStateMachine.change_state
function GameStateMachine:change_state(state, params)
	if _G.GameStateFilters.any_ingame_playing[state:name()] then
		Perfometer:start()

	elseif _G.GameStateFilters.any_end_game[state:name()] then
		Perfometer:stop()
		Perfometer:export()
	end

	pm_original_gamestatemachine_changestate(self, state, params)
end
